﻿local L = AceLibrary("AceLocale-2.2"):new("SacredBuff")
local BS = AceLibrary("Babble-Spell-2.2")
local _G = getfenv(0)
local _, _, rev = string.find("$Rev: 19848 $", "(%d+)")
SacredBuff.fileRevisions["UI.lua"] = tonumber(rev)
rev = nil

local frameNum = 1

local stateNames = {
	"singletarget",
	"grouptarget",
	"singleplayer",
	"groupplayer"
}

local hearthstone = GetItemInfo(6948)

function SacredBuff:CreateTooltip()
	local f = CreateFrame("GameTooltip", "SacredBuffTooltip", UIParent, "GameTooltipTemplate")
	f:SetScript("OnLoad",function() this:SetOwner(WorldFrame, "ANCHOR_NONE") end)
	f:SetFrameStrata("LOW")
	f:SetToplevel(true)
	f:SetMovable(true)
	f:SetClampedToScreen(true)
	self.tooltip = f
end

function SacredBuff:CreateMainFrame()
	local f = CreateFrame("Frame", "SacredBuffFrame", UIParent, "SecureStateHeaderTemplate")
	f:Hide()
	f:EnableMouse(true)
	f:SetMovable(true)
	f:SetHeight(128)
	f:SetWidth(128)
	f:SetFrameStrata("LOW")
	f:SetClampedToScreen(true)
	
	f.title = f:CreateFontString(nil, "ARTWORK")
	f.title:ClearAllPoints()
	f.title:SetPoint("BOTTOM", f, "TOP", 0, 0)
	f.title:SetFont("Fonts\\ARIALN.ttf", 11, "OUTLINE")
	f.title:SetText(L["SacredBuff"])
	f.title:SetTextColor(1.00, 1.00, 1.00, 1.00)
	
	f:SetBackdrop({ 
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
		edgeFile = "", tile = true, tileSize = 16, edgeSize = 16, 
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	})
	if ( self.db.profile.movable ) then
		f:SetBackdropColor(0.00, 0.00, 0.00, 0.60)
	else
		f:SetBackdropColor(0.00, 0.00, 0.00, 0.00)
		f.title:Hide()
	end
	
	f:SetAttribute("statebutton1", "0:singletarget;1:grouptarget")
	f:SetAttribute("statebutton2", "0:singleplayer;1:groupplayer")
	f:SetAttribute("state", self.db.char.groupMode)
	
	
	f:SetAttribute("unit-singletarget", ATTRIBUTE_NOOP)
	f:SetAttribute("unit-grouptarget", ATTRIBUTE_NOOP)
	f:SetAttribute("unit-singleplayer", "player")
	f:SetAttribute("unit-groupplayer", "player")
	
	f.StateChanged = function()
		self.db.char.groupMode = f:GetAttribute("state")
		self:TriggerEvent("SacredBuff_ModeChanged")
	end
	
	self:RestorePosition()
	self:ScaleFrame()
	
	f:SetScript("OnMouseDown", function()
		if (( arg1 == "LeftButton" ) and ( self.db.profile.movable == true )) then
			f:StartMoving()
			f.isMoving = true
		end
	end)
	f:SetScript("OnMouseUp", function()
		if ( this.isMoving ) then
			f:StopMovingOrSizing()
			self:SavePosition()
			f.isMoving = false
		end
	end)
	f:SetScript("OnHide", function()
		if ( this.isMoving ) then
			f:StopMovingOrSizing()
			f.isMoving = false
		end
	end)
	
	f.main = CreateFrame("Button", "SacredBuffFrameMain", f, "SecureActionButtonTemplate")
	f.main:SetWidth(64)
	f.main:SetHeight(64)
	f.main:EnableMouse(true)
	f.main:SetFrameStrata("LOW")
	
	f.main:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	
	f.main:SetFont("Fonts\\ARIALN.ttf", 16, "OUTLINE")
	f.main:SetHighlightTextColor(1, 1, 0, 1)
	f.main:SetText(self.db.char.sacredCount)
	f.main:SetTextColor(1.00, 1.00, 1.00, 1.00)
	f.main:ClearAllPoints()
	f.main:SetPoint("CENTER", f, "CENTER", 0, 0)
	
	f.main:SetNormalTexture("Interface\\Addons\\SacredBuff\\UI\\Symbol32")
	
	f.main:SetAttribute("newstate1", "0,1")
	f.main:SetAttribute("*type2", "item")
	f.main:SetAttribute("item", hearthstone)
	f:SetAttribute("addchild", f.main)
	
	local t = f:CreateTexture(nil, "OVERLAY")
	t:SetWidth(64)
	t:SetHeight(64)
	t:SetTexture("Interface\\AddOns\\SacredBuff\\UI\\serenity0")
	t:SetPoint("CENTER", f.main, "CENTER", 0, 0)
	f.main.background = t
	
	self.frame = f
	
	self:UpdateCandleCount()
end

function SacredBuff:ScaleFrame()
	self:SavePosition()
	_G["SacredBuffFrame"]:SetScale(self.db.profile.scaleSize)
	self:RestorePosition()
end

function SacredBuff:SavePosition()
	local f = _G["SacredBuffFrame"]
	local x, y = f:GetLeft(), f:GetTop()
	local s = f:GetEffectiveScale()
	x, y = ( x * s ), ( y * s )
	self.db.profile.PosX = x
	self.db.profile.PosY = y
end

function SacredBuff:RestorePosition()
	local x = SacredBuff.db.profile.PosX or 0
	local y = SacredBuff.db.profile.PosY or 0
	local f = _G["SacredBuffFrame"]
	local s = f:GetEffectiveScale()
	x, y = ( x / s ), ( y / s )
	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end

function SacredBuff:NewButton(x, y, singleSpell, partySpell)
	local frameName = "SacredBuffButton"..frameNum
	local f = CreateFrame("Button", frameName, self.frame.main, "SecureActionButtonTemplate")
	f:SetWidth(35)
	f:SetHeight(35)
	f:EnableMouse(true)
	f:SetFrameStrata("MEDIUM")
	f:SetToplevel(true)
	f:SetNormalTexture("Interface\\AddOns\\SacredBuff\\UI\\serenity0")
	f:SetHighlightTexture("Interface\\AddOns\\SacredBuff\\UI\\serenity0")
	f:SetPushedTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	f:RegisterForClicks("AnyUp")
	
	f:ClearAllPoints()
	f:SetPoint("CENTER", self.frame.main, "CENTER", tonumber(x), tonumber(y))
	
	f:SetScript("OnLeave", function()
		GameTooltip:Hide()
		GameTooltip:ClearLines()
	end)
	
	local t = f:CreateTexture(frameName.."Icon", "BACKGROUND")
	t:SetDrawLayer("BACKGROUND")
	t:SetWidth(26)
	t:SetHeight(26)
	t:ClearAllPoints()
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	f.icon = t
	
	t = f:CreateTexture(frameName.."IconOverlay", "OVERLAY")
	t:SetWidth(40)
	t:SetHeight(40)
	t:SetTexture("Interface\\AddOns\\SacredBuff\\UI\\serenity0")
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	f.icon.overlay = t
	
	t = f:GetNormalTexture()
	t:SetWidth(40)
	t:SetHeight(40)
	t:SetDrawLayer("BACKGROUND")
	t:ClearAllPoints()
	t:SetPoint("CENTER", f.icon, "CENTER", 0, 0)
	f:SetNormalTexture(t)
	f.normal = t
	
	t = f:GetHighlightTexture()
	t:SetVertexColor( 75/255, 216/255, 241/255 )
	t:SetWidth(40)
	t:SetHeight(40)
	t:ClearAllPoints()
	t:SetBlendMode("BLEND")
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	f:SetHighlightTexture(t)
	f.highlight = t
	
	t = f:GetPushedTexture()
	t:SetWidth(35)
	t:SetHeight(35)
	t:ClearAllPoints()
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	t:SetDrawLayer("HIGHLIGHT")
	t:SetBlendMode("ADD")
	f:SetPushedTexture(t)
	f.pushed = t
	
	self.frame:SetAttribute("addchild", f)
	f:SetAttribute("useparent-statebutton", true)
	f:SetAttribute("useparent-unit", true)
	if ( singleSpell ) then
		f:SetAttribute("*type*", "spell")
		f:SetAttribute("*spell-singletarget", singleSpell)
		f:SetAttribute("*spell-singleplayer", singleSpell)
		f.states = { singleSpell }
		f:SetAttribute("*spell-grouptarget", partySpell or singleSpell)
		f:SetAttribute("*spell-groupplayer", partySpell or singleSpell)
		table.insert(f.states, partySpell or singleSpell)
		f.icon:SetTexture( BS:GetSpellIcon(f.states[( self.db.char.groupMode + 1 )]) )
	end
	
	if ( not self:IsActive() ) then
		f:Hide()
	end
	
	frameNum = frameNum + 1
	return f
end

function SacredBuff:UpdateButtons()
	if ( self.db.char.playerLevel > 39 ) then
		if ( not self.frame.mount ) then
			local f = self:NewButton(-32, -32)
			f:SetAttribute("type", "item")
			self.frame.mount = f
		end
		self:UpdateMountLocs()
		local tbl
		if ( GetRealZoneText() == L["Ahn'Qiraj"] ) then
			tbl = "prefAQMount"
		else
			tbl = "prefMount"
		end
		if ( self.db.char[tbl] ) then
			local bag, slot = self.db.char[tbl][2], self.db.char[tbl][3]
			self.frame.mount.icon:SetTexture( GetContainerItemInfo(bag, slot) )
			self.frame.mount:SetAttribute("bag", bag)
			self.frame.mount:SetAttribute("slot", slot)
			self.frame.mount:Show()
		else
			self.frame.mount:Hide()
		end
	end
	
	if ( self.db.char.spells[ BS["Inner Fire"] ] ) then
		if ( not self.frame[ BS["Inner Fire"] ] ) then
			self.frame[ BS["Inner Fire"] ] = self:NewButton(32, -32, BS["Inner Fire"])
		end
	end
	
	if ( self.db.char.spells[ BS["Resurrection"] ] ) then
		if ( not self.frame[ BS["Resurrection"] ] ) then
			self.frame[ BS["Resurrection"] ] = self:NewButton(0, -45, BS["Resurrection"])
		end
	end
	
	if ( self.db.char.spells["RACIAL"] ) then
		if ( not self.frame["RACIAL"] ) then
			self.frame["RACIAL"] = self:NewButton(45, 0, self.db.char.spells["RACIAL"][4])
		end
	end
	
	if ( self.db.char.spells["TALENT"] ) then
		if ( not self.frame["TALENT"] ) then
			self.frame["TALENT"] = self:NewButton(-45, 0, self.db.char.spells["TALENT"][4])
		end
	end
	if ( self.db.char.spells[ BS["Divine Spirit"] ] ) then
		if ( not self.frame[ BS["Divine Spirit"] ] ) then
			self.frame[ BS["Divine Spirit"] ] = self:NewButton(-32, 32, BS["Divine Spirit"], BS["Prayer of Spirit"])
		end
	end
	
	if ( self.db.char.spells[ BS["Power Word: Fortitude"] ] ) then
		if ( not self.frame[ BS["Power Word: Fortitude"] ] ) then
			self.frame[ BS["Power Word: Fortitude"] ] = self:NewButton(0, 45, BS["Power Word: Fortitude"], BS["Prayer of Fortitude"])
		end
	end
	
	if ( self.db.char.spells[ BS["Shadow Protection"] ] ) then
		if ( not self.frame[ BS["Shadow Protection"] ] ) then
			self.frame[ BS["Shadow Protection"] ] = self:NewButton(32, 32, BS["Shadow Protection"], BS["Prayer of Shadow Protection"])
		end
	end
	
end